from __future__ import annotations

from pathlib import Path
from typing import Any

from recite_mcp.config import Settings
from recite_mcp.models import ReceiptRecord
from recite_mcp.tools import ReciteTools


# ---------------------------------------------------------------------------
# Stub API client — returns kwargs so tests can inspect forwarded args
# ---------------------------------------------------------------------------

class _Client:
    """Minimal stub that records and echoes calls."""

    def process_receipt(self, _path: Path) -> ReceiptRecord:
        return ReceiptRecord(vendor="Bakery", date="2026-02-22", total=8.0, tax=0.8, currency="USD", category="Meals")

    def scan_receipt(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def get_scan(self, scan_id: str) -> dict:
        return {"scan_id": scan_id}

    def create_transaction(self, transaction: dict) -> dict:
        return {"transaction_id": "txn_new", **transaction}

    def list_transactions(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def get_transaction(self, transaction_id: str) -> dict:
        return {"transaction_id": transaction_id}

    def update_transaction(self, transaction_id: str, changes: dict) -> dict:
        return {"transaction_id": transaction_id, "changes": changes}

    def delete_transaction(self, transaction_id: str) -> dict:
        return {"status": "deleted", "transaction_id": transaction_id}

    def import_transactions(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def submit_batch_scans(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def get_batch_scan_status(self, job_id: str) -> dict:
        return {"job_id": job_id, "status": "completed"}

    def get_batch_scan_results(self, job_id: str) -> dict:
        return {"job_id": job_id, "results": []}

    def list_projects(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def create_project(self, *, name: str, description: str | None = None) -> dict:
        return {"project_id": "proj_new", "name": name}

    def update_project(self, project_id: str, **kwargs: Any) -> dict:
        return {"project_id": project_id, **kwargs}

    def delete_project(self, project_id: str) -> dict:
        return {"status": "deleted", "project_id": project_id}

    def get_summary(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def create_webhook(self, *, url: str, events: list[str], secret: str | None = None) -> dict:
        return {"webhook_id": "wh_new", "url": url, "events": events}

    def list_webhooks(self) -> dict:
        return {"webhooks": []}

    def delete_webhook(self, webhook_id: str) -> dict:
        return {"status": "deleted", "webhook_id": webhook_id}

    def create_rule(self, *, rule_type: str, condition: dict, action: dict, priority: int | None = None) -> dict:
        return {"rule_id": "rule_new", "rule_type": rule_type}

    def list_rules(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def delete_rule(self, rule_id: str) -> dict:
        return {"status": "deleted", "rule_id": rule_id}

    def get_usage(self, **kwargs: Any) -> dict:
        return {"data": kwargs}

    def export_transactions(self, *, format: str, filters: dict | None = None) -> dict:
        return {"format": format, "filters": filters}


def _settings(tmp_path: Path) -> Settings:
    return Settings(recite_home=tmp_path, api_key="x", api_base_url="https://example", request_timeout_sec=20)


def _tools(tmp_path: Path) -> ReciteTools:
    return ReciteTools.from_settings(_settings(tmp_path), api_client=_Client())


# ---------------------------------------------------------------------------
# Legacy process_receipt / batch
# ---------------------------------------------------------------------------

def test_process_receipt_tool_writes_ledger(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    tools = ReciteTools.from_settings(_settings(tmp_path), api_client=_Client())

    result = tools.process_receipt(file_path=str(image), rename=False, dry_run=False)

    assert result.status == "ok"
    assert result.ledger_entry is not None
    assert result.ledger_entry.vendor == "Bakery"


def test_batch_dry_run_returns_preview(tmp_path: Path) -> None:
    (tmp_path / "a.jpg").write_bytes(b"a")
    (tmp_path / "b.jpg").write_bytes(b"b")
    tools = ReciteTools.from_settings(_settings(tmp_path), api_client=_Client())

    batch = tools.process_receipts_batch(input_dir=str(tmp_path), dry_run=True)

    assert batch.status == "ok"
    assert batch.processed == 0
    assert batch.preview_count == 2


# ---------------------------------------------------------------------------
# scan_receipt
# ---------------------------------------------------------------------------

def test_scan_receipt_tool_forwards_ephemeral_request(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    tools = _tools(tmp_path)

    result = tools.scan_receipt(file_path=str(image), ephemeral=True)

    assert result["data"]["ephemeral"] is True
    assert result["data"]["file_path"] == str(image)


def test_scan_receipt_tool_forwards_auto_save_params(tmp_path: Path) -> None:
    tools = _tools(tmp_path)

    result = tools.scan_receipt(
        raw_text="receipt", auto_save=True, project_id="proj_x", save_threshold="medium"
    )

    assert result["data"]["auto_save"] is True
    assert result["data"]["project_id"] == "proj_x"
    assert result["data"]["save_threshold"] == "medium"


def test_scan_receipt_tool_forwards_metadata(tmp_path: Path) -> None:
    tools = _tools(tmp_path)

    result = tools.scan_receipt(image_url="https://example.com/r.jpg", metadata={"source": "email"})

    assert result["data"]["metadata"] == {"source": "email"}


# ---------------------------------------------------------------------------
# get_scan
# ---------------------------------------------------------------------------

def test_get_scan_tool_returns_scan(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_scan("scan_abc")
    assert result["scan_id"] == "scan_abc"


# ---------------------------------------------------------------------------
# transactions
# ---------------------------------------------------------------------------

def test_create_transaction_tool_forwards_payload(tmp_path: Path) -> None:
    txn = {"date": "2026-03-01", "amount": 50.0, "transaction_type": "Expense", "category": "Food", "payment_method": "Card"}
    result = _tools(tmp_path).create_transaction(txn)
    assert result["transaction_id"] == "txn_new"
    assert result["amount"] == 50.0


def test_list_transactions_tool_forwards_filters(tmp_path: Path) -> None:
    result = _tools(tmp_path).list_transactions(start_date="2026-01-01", limit=25)
    assert result["data"]["start_date"] == "2026-01-01"
    assert result["data"]["limit"] == 25


def test_get_transaction_tool_returns_transaction(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_transaction("txn_123")
    assert result["transaction_id"] == "txn_123"


def test_update_transaction_tool_forwards_changes(tmp_path: Path) -> None:
    result = _tools(tmp_path).update_transaction("txn_123", {"amount": 99.0})
    assert result["transaction_id"] == "txn_123"
    assert result["changes"] == {"amount": 99.0}


def test_delete_transaction_tool_returns_deleted(tmp_path: Path) -> None:
    result = _tools(tmp_path).delete_transaction("txn_123")
    assert result == {"status": "deleted", "transaction_id": "txn_123"}


# ---------------------------------------------------------------------------
# import_transactions
# ---------------------------------------------------------------------------

def test_import_transactions_tool_forwards_json_payload(tmp_path: Path) -> None:
    txns = [{"date": "2026-03-01", "amount": 10.0}]
    result = _tools(tmp_path).import_transactions(transactions=txns, project_id="proj_x")
    assert result["data"]["transactions"] == txns
    assert result["data"]["project_id"] == "proj_x"


def test_import_transactions_tool_forwards_csv_text(tmp_path: Path) -> None:
    result = _tools(tmp_path).import_transactions(csv_text="date,amount\n2026-03-01,10.0")
    assert result["data"]["csv_text"].startswith("date,amount")


def test_import_transactions_tool_forwards_csv_file_path(tmp_path: Path) -> None:
    csv_file = tmp_path / "data.csv"
    csv_file.write_text("date,amount\n2026-03-01,10.0")
    result = _tools(tmp_path).import_transactions(csv_file_path=str(csv_file))
    assert result["data"]["csv_file_path"] == str(csv_file)


# ---------------------------------------------------------------------------
# submit_batch_scans
# ---------------------------------------------------------------------------

def test_submit_batch_scans_tool_forwards_items(tmp_path: Path) -> None:
    items = [{"image_url": "https://example.com/r.jpg"}]
    result = _tools(tmp_path).submit_batch_scans(items=items, auto_save=True, project_id="proj_x")
    assert result["data"]["items"] == items
    assert result["data"]["auto_save"] is True


def test_submit_batch_scans_tool_forwards_webhook(tmp_path: Path) -> None:
    result = _tools(tmp_path).submit_batch_scans(
        items=[{"image_url": "https://example.com/r.jpg"}],
        webhook_url="https://my.app/hook",
        webhook_secret="secret",
    )
    assert result["data"]["webhook_url"] == "https://my.app/hook"


# ---------------------------------------------------------------------------
# batch status / results
# ---------------------------------------------------------------------------

def test_get_batch_scan_status_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_batch_scan_status("job_1")
    assert result["job_id"] == "job_1"
    assert result["status"] == "completed"


def test_get_batch_scan_results_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_batch_scan_results("job_1")
    assert result["results"] == []


# ---------------------------------------------------------------------------
# projects
# ---------------------------------------------------------------------------

def test_list_projects_tool_forwards_params(tmp_path: Path) -> None:
    result = _tools(tmp_path).list_projects(status="active", limit=10)
    assert result["data"]["status"] == "active"
    assert result["data"]["limit"] == 10


def test_create_project_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).create_project(name="New Project", description="desc")
    assert result["project_id"] == "proj_new"
    assert result["name"] == "New Project"


def test_update_project_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).update_project("proj_1", name="Renamed", status="archived")
    assert result["project_id"] == "proj_1"


def test_delete_project_tool_returns_deleted(tmp_path: Path) -> None:
    result = _tools(tmp_path).delete_project("proj_1")
    assert result == {"status": "deleted", "project_id": "proj_1"}


# ---------------------------------------------------------------------------
# summary
# ---------------------------------------------------------------------------

def test_get_summary_tool_passes_grouping_options(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_summary(period="last_30_days", group_by="category")
    assert result["data"] == {"period": "last_30_days", "group_by": "category"}


def test_get_summary_tool_omits_none_params(tmp_path: Path) -> None:
    """None-valued params should not be forwarded to the API client."""
    result = _tools(tmp_path).get_summary(period="current_month")
    # Only non-None values should be present
    assert "group_by" not in result["data"]
    assert result["data"]["period"] == "current_month"


# ---------------------------------------------------------------------------
# webhooks
# ---------------------------------------------------------------------------

def test_create_webhook_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).create_webhook(url="https://my.app/wh", events=["batch.completed"])
    assert result["webhook_id"] == "wh_new"
    assert result["events"] == ["batch.completed"]


def test_list_webhooks_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).list_webhooks()
    assert "webhooks" in result


def test_delete_webhook_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).delete_webhook("wh_1")
    assert result == {"status": "deleted", "webhook_id": "wh_1"}


# ---------------------------------------------------------------------------
# rules
# ---------------------------------------------------------------------------

def test_create_rule_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).create_rule(
        rule_type="vendor_category",
        condition={"vendor": "Starbucks"},
        action={"set_category": "Coffee"},
        priority=1,
    )
    assert result["rule_id"] == "rule_new"
    assert result["rule_type"] == "vendor_category"


def test_list_rules_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).list_rules(limit=10)
    assert result["data"]["limit"] == 10


def test_delete_rule_tool(tmp_path: Path) -> None:
    result = _tools(tmp_path).delete_rule("rule_1")
    assert result == {"status": "deleted", "rule_id": "rule_1"}


# ---------------------------------------------------------------------------
# usage / export
# ---------------------------------------------------------------------------

def test_get_usage_tool_forwards_params(tmp_path: Path) -> None:
    result = _tools(tmp_path).get_usage(period="today", breakdown="daily")
    assert result["data"]["period"] == "today"
    assert result["data"]["breakdown"] == "daily"


def test_export_transactions_tool_forwards_format_and_filters(tmp_path: Path) -> None:
    result = _tools(tmp_path).export_transactions(format="csv", filters={"project_id": "proj_x"})
    assert result["format"] == "csv"
    assert result["filters"] == {"project_id": "proj_x"}



from __future__ import annotations

import os
from dataclasses import dataclass
from pathlib import Path

try:
    import tomllib
except ModuleNotFoundError:  # pragma: no cover
    import tomli as tomllib  # type: ignore[no-redef]


class ConfigError(RuntimeError):
    pass


@dataclass(slots=True)
class Settings:
    recite_home: Path
    api_key: str | None
    api_base_url: str
    request_timeout_sec: int

    @property
    def ledger_path(self) -> Path:
        return self.recite_home / "bookkeeping_transactions.csv"

    @property
    def memory_path(self) -> Path:
        return self.recite_home / "long_term_memory.md"


def _default_home() -> Path:
    return Path(os.environ.get("RECITE_HOME", Path.home() / ".config" / "recite")).expanduser()


def _load_config_file(home: Path) -> dict:
    config_file = home / "config.toml"
    if not config_file.exists():
        return {}
    return tomllib.loads(config_file.read_text(encoding="utf-8"))


def load_settings(require_api_key: bool = True) -> Settings:
    home = _default_home()
    home.mkdir(parents=True, exist_ok=True)
    config = _load_config_file(home)

    env_key = os.environ.get("RECITE_API_KEY")
    api_key = env_key or config.get("api_key")

    if require_api_key and not api_key:
        raise ConfigError("Missing API key. Set RECITE_API_KEY or RECITE_HOME/config.toml api_key.")

    api_base_url = str(config.get("api_base_url", "https://recite.rivra.dev/apiV1/api/v1")).rstrip("/")
    request_timeout_sec = int(config.get("request_timeout_sec", 30))

    return Settings(
        recite_home=home,
        api_key=api_key,
        api_base_url=api_base_url,
        request_timeout_sec=request_timeout_sec,
    )



from __future__ import annotations

import asyncio

from recite_mcp.server import create_server


def test_create_server_registers_tools_and_resources(tmp_path, monkeypatch) -> None:  # noqa: ANN001
    monkeypatch.setenv("RECITE_HOME", str(tmp_path))
    monkeypatch.setenv("RECITE_API_KEY", "re_test")

    server = create_server()

    if hasattr(server, "list_tools") and hasattr(server, "list_resources"):
        tool_names = {tool.name for tool in asyncio.run(server.list_tools())}
        resource_uris = {str(resource.uri) for resource in asyncio.run(server.list_resources())}
    else:
        tool_names = set(server.tools.keys())
        resource_uris = set(server.resources.keys())

    assert "process_receipt" in tool_names
    assert "process_receipts_batch" in tool_names
    assert "scan_receipt" in tool_names
    assert "get_scan" in tool_names
    assert "create_transaction" in tool_names
    assert "list_transactions" in tool_names
    assert "update_transaction" in tool_names
    assert "delete_transaction" in tool_names
    assert "import_transactions" in tool_names
    assert "submit_batch_scans" in tool_names
    assert "get_batch_scan_status" in tool_names
    assert "get_batch_scan_results" in tool_names
    assert "list_projects" in tool_names
    assert "create_project" in tool_names
    assert "update_project" in tool_names
    assert "delete_project" in tool_names
    assert "get_summary" in tool_names
    assert "create_webhook" in tool_names
    assert "list_webhooks" in tool_names
    assert "delete_webhook" in tool_names
    assert "create_rule" in tool_names
    assert "list_rules" in tool_names
    assert "delete_rule" in tool_names
    assert "get_usage" in tool_names
    assert "export_transactions" in tool_names
    assert "recite://ledger" in resource_uris
    assert "recite://health" in resource_uris


def test_create_server_does_not_require_api_key(tmp_path, monkeypatch) -> None:  # noqa: ANN001
    monkeypatch.setenv("RECITE_HOME", str(tmp_path))
    monkeypatch.delenv("RECITE_API_KEY", raising=False)

    server = create_server()

    if hasattr(server, "list_tools"):
        tool_names = {tool.name for tool in asyncio.run(server.list_tools())}
    else:
        tool_names = set(server.tools.keys())

    assert "validate_setup" in tool_names



from __future__ import annotations

from pathlib import Path

import pytest

from recite_mcp.config import ConfigError, Settings, load_settings


def test_load_settings_prefers_env_api_key(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    home = tmp_path / "recite"
    home.mkdir()
    (home / "config.toml").write_text('api_key = "from_file"\n', encoding="utf-8")
    monkeypatch.setenv("RECITE_HOME", str(home))
    monkeypatch.setenv("RECITE_API_KEY", "from_env")

    settings = load_settings()

    assert settings.api_key == "from_env"
    assert settings.recite_home == home


def test_load_settings_uses_file_api_key_when_env_missing(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    home = tmp_path / "recite"
    home.mkdir()
    (home / "config.toml").write_text('api_key = "from_file"\napi_base_url = "https://example.test"\n', encoding="utf-8")
    monkeypatch.setenv("RECITE_HOME", str(home))
    monkeypatch.delenv("RECITE_API_KEY", raising=False)

    settings = load_settings()

    assert settings.api_key == "from_file"
    assert settings.api_base_url == "https://example.test"


def test_load_settings_raises_without_api_key(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    monkeypatch.setenv("RECITE_HOME", str(tmp_path / "recite"))
    monkeypatch.delenv("RECITE_API_KEY", raising=False)

    with pytest.raises(ConfigError):
        load_settings(require_api_key=True)


def test_load_settings_allows_missing_api_key_when_not_required(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    monkeypatch.setenv("RECITE_HOME", str(tmp_path / "recite"))
    monkeypatch.delenv("RECITE_API_KEY", raising=False)

    settings = load_settings(require_api_key=False)

    assert settings.api_key is None
    assert settings.recite_home.exists()


def test_settings_paths_are_derived_from_home(tmp_path: Path) -> None:
    settings = Settings(
        recite_home=tmp_path,
        api_key="test",
        api_base_url="https://recite.rivra.dev",
        request_timeout_sec=30,
    )

    assert settings.ledger_path == tmp_path / "bookkeeping_transactions.csv"
    assert settings.memory_path == tmp_path / "long_term_memory.md"



from __future__ import annotations

from recite_mcp.memory import MemoryRepository


def test_update_and_list_memory(tmp_path) -> None:  # noqa: ANN001
    repo = MemoryRepository(tmp_path / "memory.md")

    entry = repo.add_instruction("Prefer category Meals for coffee", tags=["category", "coffee"])
    rows = repo.list_instructions()

    assert entry.instruction.startswith("Prefer")
    assert rows[0].tags == ["category", "coffee"]




from __future__ import annotations

from dataclasses import asdict
from pathlib import Path
from typing import Any

from recite_mcp.api_client import ApiClient
from recite_mcp.config import Settings
from recite_mcp.ledger import LedgerRepository
from recite_mcp.memory import MemoryRepository
from recite_mcp.models import BatchProcessResult, ProcessResult

_DEFAULT_EXTENSIONS = (".png", ".jpg", ".jpeg", ".pdf")


class ReciteTools:
    def __init__(
        self,
        settings: Settings,
        api_client: ApiClient,
        ledger: LedgerRepository,
        memory: MemoryRepository,
    ) -> None:
        self._settings = settings
        self._api_client = api_client
        self._ledger = ledger
        self._memory = memory

    @classmethod
    def from_settings(cls, settings: Settings, api_client: ApiClient | object | None = None) -> "ReciteTools":
        resolved_client = api_client if api_client is not None else ApiClient(settings)
        return cls(
            settings=settings,
            api_client=resolved_client,  # type: ignore[arg-type]
            ledger=LedgerRepository(settings.ledger_path),
            memory=MemoryRepository(settings.memory_path),
        )

    def process_receipt(
        self,
        file_path: str,
        rename: bool = False,
        category_hint: str | None = None,
        dry_run: bool = False,
    ) -> ProcessResult:
        path = Path(file_path).expanduser()
        receipt = self._api_client.process_receipt(path)
        if category_hint:
            receipt.category = category_hint

        if dry_run:
            return ProcessResult(status="ok", message="dry_run", receipt=receipt)

        entry = self._ledger.append_receipt(receipt, source_file=str(path))
        renamed_to = None
        if rename:
            renamed_to = self._rename_file(path, entry.vendor, entry.date, entry.total)
        return ProcessResult(status="ok", message="processed", ledger_entry=entry, receipt=receipt, renamed_to=renamed_to)

    def scan_receipt(
        self,
        *,
        file_path: str | None = None,
        image_url: str | None = None,
        image_base64: str | None = None,
        raw_text: str | None = None,
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        status: str | None = None,
        image_type: str | None = None,
        idempotency_key: str | None = None,
        metadata: dict[str, Any] | None = None,
        ephemeral: bool = False,
    ) -> dict[str, Any]:
        return self._api_client.scan_receipt(
            file_path=file_path,
            image_url=image_url,
            image_base64=image_base64,
            raw_text=raw_text,
            auto_save=auto_save,
            save_threshold=save_threshold,
            project_id=project_id,
            status=status,
            image_type=image_type,
            idempotency_key=idempotency_key,
            metadata=metadata,
            ephemeral=ephemeral,
        )

    def get_scan(self, scan_id: str) -> dict[str, Any]:
        return self._api_client.get_scan(scan_id)

    def create_transaction(self, transaction: dict[str, Any]) -> dict[str, Any]:
        return self._api_client.create_transaction(transaction)

    def list_transactions(
        self,
        *,
        start_date: str | None = None,
        end_date: str | None = None,
        transaction_type: str | None = None,
        category: str | None = None,
        vendor: str | None = None,
        payment_method: str | None = None,
        amount_min: float | int | None = None,
        amount_max: float | int | None = None,
        status: str | None = None,
        project_id: str | None = None,
        source: str | None = None,
        agent_name: str | None = None,
        sort_by: str | None = None,
        sort_order: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict[str, Any]:
        return self._api_client.list_transactions(
            start_date=start_date,
            end_date=end_date,
            transaction_type=transaction_type,
            category=category,
            vendor=vendor,
            payment_method=payment_method,
            amount_min=amount_min,
            amount_max=amount_max,
            status=status,
            project_id=project_id,
            source=source,
            agent_name=agent_name,
            sort_by=sort_by,
            sort_order=sort_order,
            limit=limit,
            offset=offset,
            format=format,
        )

    def get_transaction(self, transaction_id: str) -> dict[str, Any]:
        return self._api_client.get_transaction(transaction_id)

    def update_transaction(self, transaction_id: str, changes: dict[str, Any]) -> dict[str, Any]:
        return self._api_client.update_transaction(transaction_id, changes)

    def delete_transaction(self, transaction_id: str) -> dict[str, Any]:
        return self._api_client.delete_transaction(transaction_id)

    def import_transactions(
        self,
        *,
        transactions: list[dict[str, Any]] | None = None,
        csv_text: str | None = None,
        csv_file_path: str | None = None,
        all_or_nothing: bool | None = None,
        project_id: str | None = None,
    ) -> dict[str, Any]:
        return self._api_client.import_transactions(
            transactions=transactions,
            csv_text=csv_text,
            csv_file_path=csv_file_path,
            all_or_nothing=all_or_nothing,
            project_id=project_id,
        )

    def submit_batch_scans(
        self,
        *,
        items: list[dict[str, Any]],
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        webhook_url: str | None = None,
        webhook_secret: str | None = None,
    ) -> dict[str, Any]:
        return self._api_client.submit_batch_scans(
            items=items,
            auto_save=auto_save,
            save_threshold=save_threshold,
            project_id=project_id,
            webhook_url=webhook_url,
            webhook_secret=webhook_secret,
        )

    def get_batch_scan_status(self, job_id: str) -> dict[str, Any]:
        return self._api_client.get_batch_scan_status(job_id)

    def get_batch_scan_results(self, job_id: str) -> dict[str, Any]:
        return self._api_client.get_batch_scan_results(job_id)

    def list_projects(
        self,
        *,
        status: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict[str, Any]:
        return self._api_client.list_projects(status=status, limit=limit, offset=offset, format=format)

    def create_project(self, name: str, description: str | None = None) -> dict[str, Any]:
        return self._api_client.create_project(name=name, description=description)

    def update_project(
        self,
        project_id: str,
        *,
        name: str | None = None,
        description: str | None = None,
        status: str | None = None,
    ) -> dict[str, Any]:
        return self._api_client.update_project(project_id, name=name, description=description, status=status)

    def delete_project(self, project_id: str) -> dict[str, Any]:
        return self._api_client.delete_project(project_id)

    def get_summary(
        self,
        *,
        period: str | None = None,
        start_date: str | None = None,
        end_date: str | None = None,
        project_id: str | None = None,
        group_by: str | None = None,
    ) -> dict[str, Any]:
        kwargs = {
            "period": period,
            "start_date": start_date,
            "end_date": end_date,
            "project_id": project_id,
            "group_by": group_by,
        }
        return self._api_client.get_summary(**{key: value for key, value in kwargs.items() if value is not None})

    def create_webhook(self, url: str, events: list[str], secret: str | None = None) -> dict[str, Any]:
        return self._api_client.create_webhook(url=url, events=events, secret=secret)

    def list_webhooks(self) -> dict[str, Any]:
        return self._api_client.list_webhooks()

    def delete_webhook(self, webhook_id: str) -> dict[str, Any]:
        return self._api_client.delete_webhook(webhook_id)

    def create_rule(
        self,
        *,
        rule_type: str,
        condition: dict[str, Any],
        action: dict[str, Any],
        priority: int | None = None,
    ) -> dict[str, Any]:
        return self._api_client.create_rule(rule_type=rule_type, condition=condition, action=action, priority=priority)

    def list_rules(self, *, limit: int | None = None, offset: int | None = None) -> dict[str, Any]:
        return self._api_client.list_rules(limit=limit, offset=offset)

    def delete_rule(self, rule_id: str) -> dict[str, Any]:
        return self._api_client.delete_rule(rule_id)

    def get_usage(self, *, period: str | None = None, breakdown: str | None = None) -> dict[str, Any]:
        return self._api_client.get_usage(period=period, breakdown=breakdown)

    def export_transactions(self, *, format: str, filters: dict[str, Any] | None = None) -> dict[str, Any]:
        return self._api_client.export_transactions(format=format, filters=filters)

    def process_receipts_batch(
        self,
        input_dir: str,
        rename: bool = False,
        dry_run: bool = True,
        recursive: bool = True,
    ) -> BatchProcessResult:
        base = Path(input_dir).expanduser()
        iterator = base.rglob("*") if recursive else base.glob("*")
        files = [p for p in iterator if p.is_file() and p.suffix.lower() in _DEFAULT_EXTENSIONS]

        if dry_run:
            return BatchProcessResult(
                status="ok",
                processed=0,
                failed=0,
                preview_count=len(files),
                items=[{"file": str(p), "status": "preview"} for p in files],
            )

        items: list[dict] = []
        processed = 0
        failed = 0
        for path in files:
            try:
                result = self.process_receipt(str(path), rename=rename, dry_run=False)
                items.append(
                    {
                        "file": str(path),
                        "status": "ok",
                        "entry_id": result.ledger_entry.entry_id if result.ledger_entry else None,
                    }
                )
                processed += 1
            except Exception as exc:  # noqa: BLE001
                failed += 1
                items.append({"file": str(path), "status": "error", "error": str(exc)})

        return BatchProcessResult(status="ok", processed=processed, failed=failed, preview_count=len(files), items=items)

    def update_memory(self, instruction: str, tags: list[str] | None = None) -> dict:
        return asdict(self._memory.add_instruction(instruction, tags=tags))

    def list_memory(self) -> list[dict]:
        return [asdict(entry) for entry in self._memory.list_instructions()]

    def add_ledger_correction(self, original_entry_id: str, corrected_fields: dict, reason: str) -> dict:
        return asdict(self._ledger.add_correction(original_entry_id, corrected_fields=corrected_fields, reason=reason))

    def summarize_ledger(self, group_by: str = "vendor") -> dict:
        return self._ledger.summarize(group_by=group_by)

    def export_ledger(self, format: str, output_path: str) -> dict:
        output = Path(output_path).expanduser()
        if format == "csv":
            file_path = self._ledger.export_csv(output)
        elif format == "json":
            file_path = self._ledger.export_json(output)
        else:
            raise ValueError(f"Unsupported format: {format}")
        return {"status": "ok", "path": str(file_path)}

    @staticmethod
    def _rename_file(path: Path, vendor: str, date: str, total: float) -> str:
        safe_vendor = "".join(ch for ch in vendor if ch.isalnum() or ch in ("-", "_")).strip() or "vendor"
        target = path.with_name(f"{date}_{safe_vendor}_{total:.2f}{path.suffix.lower()}")
        if target == path:
            return str(target)
        path.rename(target)
        return str(target)



from __future__ import annotations

from dataclasses import asdict

from recite_mcp.config import Settings
from recite_mcp.ledger import LedgerRepository


class ResourceProvider:
    def __init__(self, settings: Settings) -> None:
        self._settings = settings
        self._ledger = LedgerRepository(settings.ledger_path)

    def get_ledger_rows(self) -> list[dict]:
        return [asdict(row) for row in self._ledger.read_all()]

    def get_memory_text(self) -> str:
        if not self._settings.memory_path.exists():
            return ""
        return self._settings.memory_path.read_text(encoding="utf-8")

    def get_health(self) -> dict:
        issues: list[str] = []
        if not self._settings.api_key:
            issues.append("missing_api_key")
        status = "ok" if not issues else "degraded"
        return {
            "status": status,
            "recite_home": str(self._settings.recite_home),
            "ledger_path": str(self._settings.ledger_path),
            "memory_path": str(self._settings.memory_path),
            "has_api_key": bool(self._settings.api_key),
            "issues": issues,
        }



from __future__ import annotations

import json
from dataclasses import asdict
from pathlib import Path

from recite_mcp.models import MemoryEntry, utc_now_iso


class MemoryRepository:
    def __init__(self, path: Path) -> None:
        self.path = path
        self.path.parent.mkdir(parents=True, exist_ok=True)

    def _ensure_file(self) -> None:
        if not self.path.exists():
            self.path.write_text("", encoding="utf-8")

    def add_instruction(self, instruction: str, tags: list[str] | None = None) -> MemoryEntry:
        self._ensure_file()
        entry = MemoryEntry(timestamp_utc=utc_now_iso(), instruction=instruction, tags=tags or [])
        with self.path.open("a", encoding="utf-8") as f:
            f.write(json.dumps(asdict(entry)) + "\n")
        return entry

    def list_instructions(self) -> list[MemoryEntry]:
        self._ensure_file()
        entries: list[MemoryEntry] = []
        for line in self.path.read_text(encoding="utf-8").splitlines():
            if not line.strip():
                continue
            payload = json.loads(line)
            entries.append(
                MemoryEntry(
                    timestamp_utc=str(payload["timestamp_utc"]),
                    instruction=str(payload["instruction"]),
                    tags=list(payload.get("tags", [])),
                )
            )
        return entries



from __future__ import annotations

import base64
import mimetypes
from pathlib import Path
from typing import Any
from urllib.parse import quote

import requests

from recite_mcp.config import Settings
from recite_mcp.models import ReceiptRecord


class ApiClientError(RuntimeError):
    pass


class ApiClient:
    def __init__(self, settings: Settings, session: object | None = None) -> None:
        self._settings = settings
        self._session = session if session is not None else requests.Session()

    def process_receipt(self, file_path: Path) -> ReceiptRecord:
        data = self.scan_receipt(file_path=file_path, auto_save=False)
        if not isinstance(data, dict):
            raise ApiClientError(f"Invalid response payload: {data}")

        extracted = data.get("extracted_data", data)
        if not isinstance(extracted, dict):
            raise ApiClientError(f"Invalid response payload: {data}")

        vendor = _pick_first(extracted, "vendor", "merchant_name", "merchant", "store_name")
        date = _pick_first(extracted, "date", "transaction_date", "purchase_date")
        total = _pick_first(extracted, "total", "amount", "total_amount", default=0.0)
        tax = _pick_first(extracted, "tax", "sales_tax", "tax_amount", default=0.0)
        currency = _pick_first(extracted, "currency", "currency_code", default="USD")
        category = _pick_first(extracted, "category", "category_name", default=None)

        try:
            return ReceiptRecord(
                vendor=str(vendor),
                date=str(date),
                total=float(total),
                tax=float(tax),
                currency=str(currency),
                category=str(category) if category is not None else None,
            )
        except Exception as exc:  # noqa: BLE001
            raise ApiClientError(f"Invalid response payload: {data}") from exc

    def scan_receipt(
        self,
        *,
        file_path: str | Path | None = None,
        image_url: str | None = None,
        image_base64: str | None = None,
        raw_text: str | None = None,
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        status: str | None = None,
        image_type: str | None = None,
        idempotency_key: str | None = None,
        metadata: dict[str, Any] | None = None,
        ephemeral: bool = False,
    ) -> dict[str, Any]:
        """Scan a receipt using the Recite API to extract financial data.

        Provide exactly one input: file_path, image_url, image_base64, or raw_text.
        
        Args:
            file_path: Local path to an image.
            image_url: Publicly accessible URL (must use https).
            image_base64: Base64-encoded image data.
            raw_text: Pre-extracted text.
            auto_save: Auto-create a transaction if successful. Requires project_id.
            save_threshold: Confidence threshold for auto-saving.
            project_id: Project UUID. Required if auto_save is True.
            status: Target status of the transaction.
            image_type: MIME type hint for the image.
            idempotency_key: Key to prevent duplicate processing.
            metadata: Custom key-value data.
            ephemeral: Process without saving scan records server-side. Cannot be True if auto_save is True.
        """
        payload = self._build_scan_payload(
            file_path=file_path,
            image_url=image_url,
            image_base64=image_base64,
            raw_text=raw_text,
            auto_save=auto_save,
            save_threshold=save_threshold,
            project_id=project_id,
            status=status,
            image_type=image_type,
            idempotency_key=idempotency_key,
            metadata=metadata,
            ephemeral=ephemeral,
        )
        return self._request("POST", "/scan", json=payload)

    def get_scan(self, scan_id: str) -> dict[str, Any]:
        return self._request("GET", f"/scan/{_quote_path(scan_id)}")

    def create_transaction(self, transaction: dict[str, Any]) -> dict[str, Any]:
        return self._request("POST", "/transactions", json=_drop_none(transaction))

    def list_transactions(
        self,
        *,
        start_date: str | None = None,
        end_date: str | None = None,
        transaction_type: str | None = None,
        category: str | None = None,
        vendor: str | None = None,
        payment_method: str | None = None,
        amount_min: float | int | None = None,
        amount_max: float | int | None = None,
        status: str | None = None,
        project_id: str | None = None,
        source: str | None = None,
        agent_name: str | None = None,
        sort_by: str | None = None,
        sort_order: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict[str, Any]:
        params = _drop_none(
            {
                "start_date": start_date,
                "end_date": end_date,
                "transaction_type": transaction_type,
                "category": category,
                "vendor": vendor,
                "payment_method": payment_method,
                "amount_min": amount_min,
                "amount_max": amount_max,
                "status": status,
                "project_id": project_id,
                "source": source,
                "agent_name": agent_name,
                "sort_by": sort_by,
                "sort_order": sort_order,
                "limit": limit,
                "offset": offset,
                "format": format,
            }
        )
        return self._request("GET", "/transactions", params=params)

    def get_transaction(self, transaction_id: str) -> dict[str, Any]:
        return self._request("GET", f"/transactions/{_quote_path(transaction_id)}")

    def update_transaction(self, transaction_id: str, changes: dict[str, Any]) -> dict[str, Any]:
        return self._request("PATCH", f"/transactions/{_quote_path(transaction_id)}", json=_drop_none(changes))

    def delete_transaction(self, transaction_id: str) -> dict[str, Any]:
        self._request("DELETE", f"/transactions/{_quote_path(transaction_id)}")
        return {"status": "deleted", "transaction_id": transaction_id}

    def import_transactions(
        self,
        *,
        transactions: list[dict[str, Any]] | None = None,
        csv_text: str | None = None,
        csv_file_path: str | Path | None = None,
        all_or_nothing: bool | None = None,
        project_id: str | None = None,
    ) -> dict[str, Any]:
        """Import multiple transactions at once.

        Provide exactly one data source: transactions (list), csv_text, or csv_file_path.
        
        Args:
            transactions: List of transaction objects to import.
            csv_text: Raw CSV string content.
            csv_file_path: Local path to a CSV file.
            all_or_nothing: If True, fails the entire import if any transaction fails.
            project_id: Apply all transactions to this project UUID.
        """
        provided = [
            transactions is not None,
            csv_text is not None,
            csv_file_path is not None,
        ]
        if sum(provided) != 1:
            raise ApiClientError("Provide exactly one of transactions, csv_text, or csv_file_path.")

        if transactions is not None:
            payload = {"transactions": transactions}
            if all_or_nothing is not None:
                payload["all_or_nothing"] = all_or_nothing
            if project_id is not None:
                payload["project_id"] = project_id
            return self._request("POST", "/import/transactions", json=payload)

        params = _drop_none(
            {
                "all_or_nothing": _bool_query_param(all_or_nothing) if all_or_nothing is not None else None,
                "project_id": project_id,
            }
        )

        if csv_text is not None:
            return self._request(
                "POST",
                "/import/transactions",
                params=params,
                data=csv_text,
                headers={"Content-Type": "text/csv"},
            )

        if csv_file_path is None:
            raise ApiClientError("csv_file_path is missing")

        with open(Path(csv_file_path).expanduser(), "rb") as f:
            return self._request(
                "POST",
                "/import/transactions",
                params=params,
                data=f,
                headers={"Content-Type": "text/csv"},
            )

    def submit_batch_scans(
        self,
        *,
        items: list[dict[str, Any]],
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        webhook_url: str | None = None,
        webhook_secret: str | None = None,
    ) -> dict[str, Any]:
        """Submit multiple receipts for asynchronous batch processing.
        
        Args:
            items: List of 1-20 task items. Each must provide exactly one of 
                   file_path, image_url, or image_base64.
            auto_save: Auto-create transactions for successful scans.
            save_threshold: Confidence threshold for auto-saving.
            project_id: Apply all auto-saved transactions to this project UUID.
            webhook_url: URL to call when batch completes.
            webhook_secret: HMAC-SHA256 signature secret for the webhook.
        """
        normalized_items = [self._normalize_batch_item(item) for item in items]
        payload: dict[str, Any] = {"items": normalized_items, "auto_save": auto_save}
        optional_fields = {
            "save_threshold": save_threshold,
            "project_id": project_id,
            "webhook_url": webhook_url,
            "webhook_secret": webhook_secret,
        }
        payload.update(_drop_none(optional_fields))
        return self._request("POST", "/batch/scans", json=payload)

    def get_batch_scan_status(self, job_id: str) -> dict[str, Any]:
        return self._request("GET", f"/batch/scans/{_quote_path(job_id)}")

    def get_batch_scan_results(self, job_id: str) -> dict[str, Any]:
        return self._request("GET", f"/batch/scans/{_quote_path(job_id)}/results")

    def list_projects(
        self,
        *,
        status: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict[str, Any]:
        params = _drop_none({"status": status, "limit": limit, "offset": offset, "format": format})
        return self._request("GET", "/projects", params=params)

    def create_project(self, *, name: str, description: str | None = None) -> dict[str, Any]:
        return self._request("POST", "/projects", json=_drop_none({"name": name, "description": description}))

    def update_project(
        self,
        project_id: str,
        *,
        name: str | None = None,
        description: str | None = None,
        status: str | None = None,
    ) -> dict[str, Any]:
        payload = _drop_none({"name": name, "description": description, "status": status})
        return self._request("PATCH", f"/projects/{_quote_path(project_id)}", json=payload)

    def delete_project(self, project_id: str) -> dict[str, Any]:
        self._request("DELETE", f"/projects/{_quote_path(project_id)}")
        return {"status": "deleted", "project_id": project_id}

    def get_summary(
        self,
        *,
        period: str | None = None,
        start_date: str | None = None,
        end_date: str | None = None,
        project_id: str | None = None,
        group_by: str | None = None,
    ) -> dict[str, Any]:
        params = _drop_none(
            {
                "period": period,
                "start_date": start_date,
                "end_date": end_date,
                "project_id": project_id,
                "group_by": group_by,
            }
        )
        return self._request("GET", "/summary", params=params)

    def create_webhook(self, *, url: str, events: list[str], secret: str | None = None) -> dict[str, Any]:
        return self._request("POST", "/webhooks", json=_drop_none({"url": url, "events": events, "secret": secret}))

    def list_webhooks(self) -> dict[str, Any]:
        return self._request("GET", "/webhooks")

    def delete_webhook(self, webhook_id: str) -> dict[str, Any]:
        self._request("DELETE", f"/webhooks/{_quote_path(webhook_id)}")
        return {"status": "deleted", "webhook_id": webhook_id}

    def create_rule(
        self,
        *,
        rule_type: str,
        condition: dict[str, Any],
        action: dict[str, Any],
        priority: int | None = None,
    ) -> dict[str, Any]:
        payload = {"rule_type": rule_type, "condition": condition, "action": action}
        if priority is not None:
            payload["priority"] = priority
        return self._request("POST", "/rules", json=payload)

    def list_rules(self, *, limit: int | None = None, offset: int | None = None) -> dict[str, Any]:
        return self._request("GET", "/rules", params=_drop_none({"limit": limit, "offset": offset}))

    def delete_rule(self, rule_id: str) -> dict[str, Any]:
        self._request("DELETE", f"/rules/{_quote_path(rule_id)}")
        return {"status": "deleted", "rule_id": rule_id}

    def get_usage(self, *, period: str | None = None, breakdown: str | None = None) -> dict[str, Any]:
        return self._request("GET", "/usage", params=_drop_none({"period": period, "breakdown": breakdown}))

    def export_transactions(self, *, format: str, filters: dict[str, Any] | None = None) -> dict[str, Any]:
        return self._request("POST", "/export", json=_drop_none({"format": format, "filters": filters}))

    def _build_scan_payload(
        self,
        *,
        file_path: str | Path | None,
        image_url: str | None,
        image_base64: str | None,
        raw_text: str | None,
        auto_save: bool,
        save_threshold: str | None,
        project_id: str | None,
        status: str | None,
        image_type: str | None,
        idempotency_key: str | None,
        metadata: dict[str, Any] | None,
        ephemeral: bool,
    ) -> dict[str, Any]:
        provided_inputs = [
            file_path is not None,
            image_url is not None,
            image_base64 is not None,
            raw_text is not None,
        ]
        if sum(provided_inputs) != 1:
            raise ApiClientError("Provide exactly one of file_path, image_url, image_base64, or raw_text.")
        if ephemeral and auto_save:
            raise ApiClientError("ephemeral and auto_save cannot both be true.")
        if auto_save and not project_id:
            raise ApiClientError("project_id is required when auto_save is true.")
        if image_url is not None and not image_url.startswith("https://"):
            raise ApiClientError("image_url must use https://.")

        payload: dict[str, Any] = {"auto_save": auto_save, "ephemeral": ephemeral}

        if file_path is not None:
            resolved = Path(file_path).expanduser()
            encoded = self._encode_image_file(resolved)
            payload["image_base64"] = encoded
            if image_type is None:
                image_type = _guess_image_type(resolved)
        elif image_url is not None:
            payload["image_url"] = image_url
        elif image_base64 is not None:
            payload["image_base64"] = image_base64
        else:
            payload["raw_text"] = raw_text

        payload.update(
            _drop_none(
                {
                    "save_threshold": save_threshold,
                    "project_id": project_id,
                    "status": status,
                    "image_type": image_type,
                    "idempotency_key": idempotency_key,
                    "metadata": metadata,
                }
            )
        )
        return payload

    def _normalize_batch_item(self, item: dict[str, Any]) -> dict[str, Any]:
        if not isinstance(item, dict):
            raise ApiClientError("Each batch item must be an object.")

        normalized = dict(item)
        file_path = normalized.pop("file_path", None)
        image_url = normalized.get("image_url")
        image_base64 = normalized.get("image_base64")

        provided_inputs = [
            file_path is not None,
            image_url is not None,
            image_base64 is not None,
        ]
        if sum(provided_inputs) != 1:
            raise ApiClientError("Each batch item must provide exactly one of file_path, image_url, or image_base64.")

        if file_path is not None:
            resolved = Path(file_path).expanduser()
            normalized["image_base64"] = self._encode_image_file(resolved)
            normalized.setdefault("image_type", _guess_image_type(resolved))
        elif image_url is not None and not str(image_url).startswith("https://"):
            raise ApiClientError("Batch image_url values must use https://.")

        return _drop_none(normalized)

    def _encode_image_file(self, file_path: Path) -> str:
        if not file_path.exists():
            raise ApiClientError(f"Receipt file does not exist: {file_path}")
        return base64.b64encode(file_path.read_bytes()).decode("ascii")

    def _request(
        self,
        method: str,
        path: str,
        *,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        data: Any | None = None,
        headers: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        if not self._settings.api_key:
            raise ApiClientError("Missing API key.")

        request_headers = {"Authorization": f"Bearer {self._settings.api_key}"}
        if headers is not None:
            request_headers.update(headers)
        elif json is not None:
            request_headers["Content-Type"] = "application/json"

        try:
            response = self._dispatch_request(
                method=method,
                url=f"{self._settings.api_base_url}{path}",
                headers=request_headers,
                params=params,
                json=json,
                data=data,
                timeout=self._settings.request_timeout_sec,
            )
        except Exception as exc:  # noqa: BLE001
            raise ApiClientError(f"Request failed: {exc}") from exc

        if response.status_code >= 400:
            raise ApiClientError(_extract_error_message(response))

        if response.status_code == 204:
            return {"status": "ok"}
            
        content_type = str(getattr(response, "headers", {}).get("Content-Type", "")).lower()
        if not content_type or "json" in content_type:
            text = getattr(response, "text", "")
            if not text:
                return {"status": "ok"}
            try:
                payload = response.json()
            except Exception as exc:  # noqa: BLE001
                raise ApiClientError("Invalid JSON response from Recite API.") from exc
            if isinstance(payload, dict) and payload.get("success") is False:
                raise ApiClientError(_extract_error_message_from_payload(payload))

            if isinstance(payload, dict) and "data" in payload and payload.get("success") == True:
                return payload["data"]

            if isinstance(payload, dict):
                return payload
            return {"data": payload}

        return {"content_type": content_type, "body": getattr(response, "text", "")}

    def _dispatch_request(self, **kwargs: Any) -> Any:
        request = getattr(self._session, "request", None)
        if callable(request):
            res = request(**kwargs)
            if res is not None:
                return res

        method_name = str(kwargs["method"]).lower()
        fallback = getattr(self._session, method_name, None)
        if callable(fallback):
            return fallback(kwargs["url"], **{k: v for k, v in kwargs.items() if k not in ("method", "url")})
        raise ApiClientError("Configured session does not support HTTP requests.")


def _drop_none(payload: dict[str, Any]) -> dict[str, Any]:
    return {key: value for key, value in payload.items() if value is not None}


def _pick_first(payload: dict[str, Any], *keys: str, default: object | None = None) -> object | None:
    for key in keys:
        if key in payload and payload[key] not in (None, ""):
            return payload[key]
    return default


def _guess_image_type(path: Path) -> str | None:
    guessed, _ = mimetypes.guess_type(path.name)
    return guessed


def _quote_path(value: str) -> str:
    return quote(value, safe="")


def _bool_query_param(value: bool) -> str:
    return "true" if value else "false"


def _extract_error_message(response: Any) -> str:
    try:
        payload = response.json()
    except Exception:  # noqa: BLE001
        text = str(getattr(response, "text", "")).strip()
        if text:
            return f"Recite API error ({response.status_code}): {text}"
        return f"Recite API error: {response.status_code}"

    if isinstance(payload, dict):
        return _extract_error_message_from_payload(payload)
    return f"Recite API error: {response.status_code}"


def _extract_error_message_from_payload(payload: dict[str, Any]) -> str:
    error = payload.get("error")
    if isinstance(error, dict):
        return str(error.get("message") or error.get("code") or "Unknown API error")
    if error:
        return str(error)
    return "Unknown API error"



from __future__ import annotations

import argparse
import importlib.metadata
import json
import sys
from dataclasses import asdict, is_dataclass
from typing import Any, Callable

from recite_mcp.config import load_settings
from recite_mcp.resources import ResourceProvider
from recite_mcp.tools import ReciteTools

try:  # pragma: no cover - exercised only when mcp is installed in runtime.
    from mcp.server.fastmcp import FastMCP
except Exception:  # noqa: BLE001
    FastMCP = None  # type: ignore[assignment]


class _SimpleServer:
    def __init__(self) -> None:
        self.tools: dict[str, Callable[..., Any]] = {}
        self.resources: dict[str, Callable[[], Any]] = {}

    def tool(self, name: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        def _wrap(func: Callable[..., Any]) -> Callable[..., Any]:
            self.tools[name] = func
            return func

        return _wrap

    def resource(self, name: str) -> Callable[[Callable[[], Any]], Callable[[], Any]]:
        def _wrap(func: Callable[[], Any]) -> Callable[[], Any]:
            self.resources[name] = func
            return func

        return _wrap


def _serialize(value: Any) -> Any:
    if is_dataclass(value):
        return asdict(value)
    return value


def _build_handlers(server: Any, tools: ReciteTools, resources: ResourceProvider, settings: Any) -> Any:
    @server.tool("process_receipt")
    def process_receipt(file_path: str, rename: bool = False, category_hint: str | None = None, dry_run: bool = False) -> dict:
        return _serialize(tools.process_receipt(file_path=file_path, rename=rename, category_hint=category_hint, dry_run=dry_run))

    @server.tool("process_receipts_batch")
    def process_receipts_batch(input_dir: str, rename: bool = False, dry_run: bool = True, recursive: bool = True) -> dict:
        return _serialize(tools.process_receipts_batch(input_dir=input_dir, rename=rename, dry_run=dry_run, recursive=recursive))

    @server.tool("scan_receipt")
    def scan_receipt(
        file_path: str | None = None,
        image_url: str | None = None,
        image_base64: str | None = None,
        raw_text: str | None = None,
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        status: str | None = None,
        image_type: str | None = None,
        idempotency_key: str | None = None,
        metadata: dict | None = None,
        ephemeral: bool = False,
    ) -> dict:
        """Scan a receipt using the Recite API to extract financial data.

        Provide exactly one input: file_path, image_url, image_base64, or raw_text.
        
        Args:
            file_path: Local path to an image.
            image_url: Publicly accessible URL (must use https).
            image_base64: Base64-encoded image data.
            raw_text: Pre-extracted text.
            auto_save: Auto-create a transaction if successful. Requires project_id.
            save_threshold: Confidence threshold for auto-saving.
            project_id: Project UUID. Required if auto_save is True.
            status: Target status of the transaction.
            image_type: MIME type hint for the image.
            idempotency_key: Key to prevent duplicate processing.
            metadata: Custom key-value data.
            ephemeral: Process without saving scan records server-side. Cannot be True if auto_save is True.
        """
        return tools.scan_receipt(
            file_path=file_path,
            image_url=image_url,
            image_base64=image_base64,
            raw_text=raw_text,
            auto_save=auto_save,
            save_threshold=save_threshold,
            project_id=project_id,
            status=status,
            image_type=image_type,
            idempotency_key=idempotency_key,
            metadata=metadata,
            ephemeral=ephemeral,
        )

    @server.tool("get_scan")
    def get_scan(scan_id: str) -> dict:
        return tools.get_scan(scan_id)

    @server.tool("create_transaction")
    def create_transaction(transaction: dict) -> dict:
        return tools.create_transaction(transaction)

    @server.tool("list_transactions")
    def list_transactions(
        start_date: str | None = None,
        end_date: str | None = None,
        transaction_type: str | None = None,
        category: str | None = None,
        vendor: str | None = None,
        payment_method: str | None = None,
        amount_min: float | int | None = None,
        amount_max: float | int | None = None,
        status: str | None = None,
        project_id: str | None = None,
        source: str | None = None,
        agent_name: str | None = None,
        sort_by: str | None = None,
        sort_order: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict:
        return tools.list_transactions(
            start_date=start_date,
            end_date=end_date,
            transaction_type=transaction_type,
            category=category,
            vendor=vendor,
            payment_method=payment_method,
            amount_min=amount_min,
            amount_max=amount_max,
            status=status,
            project_id=project_id,
            source=source,
            agent_name=agent_name,
            sort_by=sort_by,
            sort_order=sort_order,
            limit=limit,
            offset=offset,
            format=format,
        )

    @server.tool("get_transaction")
    def get_transaction(transaction_id: str) -> dict:
        return tools.get_transaction(transaction_id)

    @server.tool("update_transaction")
    def update_transaction(transaction_id: str, changes: dict) -> dict:
        return tools.update_transaction(transaction_id, changes)

    @server.tool("delete_transaction")
    def delete_transaction(transaction_id: str) -> dict:
        return tools.delete_transaction(transaction_id)

    @server.tool("import_transactions")
    def import_transactions(
        transactions: list[dict] | None = None,
        csv_text: str | None = None,
        csv_file_path: str | None = None,
        all_or_nothing: bool | None = None,
        project_id: str | None = None,
    ) -> dict:
        """Import multiple transactions at once.

        Provide exactly one data source: transactions (list), csv_text, or csv_file_path.
        
        Args:
            transactions: List of transaction objects to import.
            csv_text: Raw CSV string content.
            csv_file_path: Local path to a CSV file.
            all_or_nothing: If True, fails the entire import if any transaction fails.
            project_id: Apply all transactions to this project UUID.
        """
        return tools.import_transactions(
            transactions=transactions,
            csv_text=csv_text,
            csv_file_path=csv_file_path,
            all_or_nothing=all_or_nothing,
            project_id=project_id,
        )

    @server.tool("submit_batch_scans")
    def submit_batch_scans(
        items: list[dict],
        auto_save: bool = False,
        save_threshold: str | None = None,
        project_id: str | None = None,
        webhook_url: str | None = None,
        webhook_secret: str | None = None,
    ) -> dict:
        """Submit multiple receipts for asynchronous batch processing.
        
        Args:
            items: List of 1-20 task items. Each must provide exactly one of 
                   file_path, image_url, or image_base64.
            auto_save: Auto-create transactions for successful scans.
            save_threshold: Confidence threshold for auto-saving.
            project_id: Apply all auto-saved transactions to this project UUID.
            webhook_url: URL to call when batch completes.
            webhook_secret: HMAC-SHA256 signature secret for the webhook.
        """
        return tools.submit_batch_scans(
            items=items,
            auto_save=auto_save,
            save_threshold=save_threshold,
            project_id=project_id,
            webhook_url=webhook_url,
            webhook_secret=webhook_secret,
        )

    @server.tool("get_batch_scan_status")
    def get_batch_scan_status(job_id: str) -> dict:
        return tools.get_batch_scan_status(job_id)

    @server.tool("get_batch_scan_results")
    def get_batch_scan_results(job_id: str) -> dict:
        return tools.get_batch_scan_results(job_id)

    @server.tool("list_projects")
    def list_projects(
        status: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        format: str | None = None,
    ) -> dict:
        return tools.list_projects(status=status, limit=limit, offset=offset, format=format)

    @server.tool("create_project")
    def create_project(name: str, description: str | None = None) -> dict:
        return tools.create_project(name=name, description=description)

    @server.tool("update_project")
    def update_project(
        project_id: str,
        name: str | None = None,
        description: str | None = None,
        status: str | None = None,
    ) -> dict:
        return tools.update_project(project_id, name=name, description=description, status=status)

    @server.tool("delete_project")
    def delete_project(project_id: str) -> dict:
        return tools.delete_project(project_id)

    @server.tool("get_summary")
    def get_summary(
        period: str | None = None,
        start_date: str | None = None,
        end_date: str | None = None,
        project_id: str | None = None,
        group_by: str | None = None,
    ) -> dict:
        return tools.get_summary(
            period=period,
            start_date=start_date,
            end_date=end_date,
            project_id=project_id,
            group_by=group_by,
        )

    @server.tool("create_webhook")
    def create_webhook(url: str, events: list[str], secret: str | None = None) -> dict:
        return tools.create_webhook(url=url, events=events, secret=secret)

    @server.tool("list_webhooks")
    def list_webhooks() -> dict:
        return tools.list_webhooks()

    @server.tool("delete_webhook")
    def delete_webhook(webhook_id: str) -> dict:
        return tools.delete_webhook(webhook_id)

    @server.tool("create_rule")
    def create_rule(rule_type: str, condition: dict, action: dict, priority: int | None = None) -> dict:
        return tools.create_rule(rule_type=rule_type, condition=condition, action=action, priority=priority)

    @server.tool("list_rules")
    def list_rules(limit: int | None = None, offset: int | None = None) -> dict:
        return tools.list_rules(limit=limit, offset=offset)

    @server.tool("delete_rule")
    def delete_rule(rule_id: str) -> dict:
        return tools.delete_rule(rule_id)

    @server.tool("get_usage")
    def get_usage(period: str | None = None, breakdown: str | None = None) -> dict:
        return tools.get_usage(period=period, breakdown=breakdown)

    @server.tool("export_transactions")
    def export_transactions(format: str, filters: dict | None = None) -> dict:
        return tools.export_transactions(format=format, filters=filters)

    @server.tool("update_memory")
    def update_memory(instruction: str, tags: list[str] | None = None) -> dict:
        return tools.update_memory(instruction=instruction, tags=tags)

    @server.tool("list_memory")
    def list_memory() -> list[dict]:
        return tools.list_memory()

    @server.tool("add_ledger_correction")
    def add_ledger_correction(original_entry_id: str, corrected_fields: dict, reason: str) -> dict:
        return tools.add_ledger_correction(original_entry_id=original_entry_id, corrected_fields=corrected_fields, reason=reason)

    @server.tool("summarize_ledger")
    def summarize_ledger(group_by: str = "vendor") -> dict:
        return tools.summarize_ledger(group_by=group_by)

    @server.tool("export_ledger")
    def export_ledger(format: str, output_path: str) -> dict:
        return tools.export_ledger(format=format, output_path=output_path)

    @server.tool("get_config")
    def get_config() -> dict:
        return {
            "recite_home": str(settings.recite_home),
            "api_base_url": settings.api_base_url,
            "request_timeout_sec": settings.request_timeout_sec,
            "has_api_key": bool(settings.api_key),
        }

    @server.tool("validate_setup")
    def validate_setup() -> dict:
        return resources.get_health()

    @server.resource("recite://ledger")
    def recite_ledger() -> list[dict]:
        return resources.get_ledger_rows()

    @server.resource("recite://memory")
    def recite_memory() -> str:
        return resources.get_memory_text()

    @server.resource("recite://health")
    def recite_health() -> dict:
        return resources.get_health()

    return server


def create_server() -> Any:
    # Don't fail hard at process start if RECITE_API_KEY is missing.
    # MCP clients should still be able to connect and call validate_setup/get_config.
    settings = load_settings(require_api_key=False)
    tools = ReciteTools.from_settings(settings)
    resources = ResourceProvider(settings)
    server = FastMCP("recite-mcp") if FastMCP is not None else _SimpleServer()
    return _build_handlers(server, tools, resources, settings)


def _build_arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog="recite-mcp", add_help=True)
    parser.add_argument(
        "--version",
        action="store_true",
        help="Print version and exit.",
    )
    parser.add_argument(
        "--validate",
        action="store_true",
        help="Print local health/config JSON and exit (does not call the Recite API).",
    )
    parser.add_argument(
        "--transport",
        choices=["stdio"],
        default="stdio",
        help="Transport to use when running as an MCP server (default: stdio).",
    )
    return parser


def main(argv: list[str] | None = None) -> None:
    args = _build_arg_parser().parse_args(argv)

    if args.version:
        try:
            version = importlib.metadata.version("recite-mcp")
        except importlib.metadata.PackageNotFoundError:
            version = "unknown"
        print(version)
        return

    if args.validate:
        settings = load_settings(require_api_key=False)
        health = ResourceProvider(settings).get_health()
        payload = {
            "config": {
                "recite_home": str(settings.recite_home),
                "api_base_url": settings.api_base_url,
                "request_timeout_sec": settings.request_timeout_sec,
                "has_api_key": bool(settings.api_key),
            },
            "health": health,
        }
        print(json.dumps(payload, indent=2))
        sys.exit(0 if settings.api_key else 1)

    server = create_server()
    if FastMCP is not None and hasattr(server, "run"):
        server.run(transport=args.transport)
        return
    # Fallback mode is primarily for local dev when mcp isn't installed.
    print("recite-mcp initialized (fallback mode)")
    print(f"tools={','.join(sorted(server.tools.keys()))}")
    print(f"resources={','.join(sorted(server.resources.keys()))}")


if __name__ == "__main__":
    main()



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone


def utc_now_iso() -> str:
    return datetime.now(tz=timezone.utc).isoformat()


@dataclass(slots=True)
class ReceiptRecord:
    vendor: str
    date: str
    total: float
    tax: float
    currency: str
    category: str | None = None


@dataclass(slots=True)
class LedgerEntry:
    entry_id: str
    timestamp_utc: str
    entry_type: str
    vendor: str
    date: str
    total: float
    tax: float
    currency: str
    category: str
    source_file: str
    ref_entry_id: str = ""
    correction_reason: str = ""


@dataclass(slots=True)
class MemoryEntry:
    timestamp_utc: str
    instruction: str
    tags: list[str] = field(default_factory=list)


@dataclass(slots=True)
class ProcessResult:
    status: str
    message: str
    ledger_entry: LedgerEntry | None = None
    receipt: ReceiptRecord | None = None
    renamed_to: str | None = None


@dataclass(slots=True)
class BatchProcessResult:
    status: str
    processed: int
    failed: int
    preview_count: int
    items: list[dict]




from __future__ import annotations

from pathlib import Path

from recite_mcp.ledger import LedgerRepository
from recite_mcp.models import ReceiptRecord


def test_append_and_read_entries(tmp_path: Path) -> None:
    repo = LedgerRepository(tmp_path / "ledger.csv")
    receipt = ReceiptRecord(vendor="Cafe", date="2026-02-22", total=12.0, tax=1.0, currency="USD", category="Meals")

    entry = repo.append_receipt(receipt, source_file="a.jpg")
    rows = repo.read_all()

    assert entry.vendor == "Cafe"
    assert len(rows) == 1
    assert rows[0].vendor == "Cafe"


def test_add_correction_creates_audit_row(tmp_path: Path) -> None:
    repo = LedgerRepository(tmp_path / "ledger.csv")
    entry = repo.append_receipt(
        ReceiptRecord(vendor="Shop", date="2026-02-21", total=20.0, tax=2.0, currency="USD", category="Office"),
        source_file="b.jpg",
    )

    correction = repo.add_correction(entry.entry_id, {"category": "Travel"}, reason="wrong category")
    rows = repo.read_all()

    assert correction.entry_type == "correction"
    assert correction.ref_entry_id == entry.entry_id
    assert len(rows) == 2


def test_summary_by_vendor(tmp_path: Path) -> None:
    repo = LedgerRepository(tmp_path / "ledger.csv")
    repo.append_receipt(
        ReceiptRecord(vendor="Store", date="2026-02-21", total=20.0, tax=2.0, currency="USD", category="Office"),
        source_file="b.jpg",
    )
    repo.append_receipt(
        ReceiptRecord(vendor="Store", date="2026-02-22", total=30.0, tax=3.0, currency="USD", category="Office"),
        source_file="c.jpg",
    )

    summary = repo.summarize(group_by="vendor")

    assert summary["Store"]["count"] == 2
    assert summary["Store"]["total"] == 50.0




from __future__ import annotations

from pathlib import Path

from recite_mcp.config import Settings
from recite_mcp.resources import ResourceProvider


def _settings(tmp_path: Path) -> Settings:
    return Settings(recite_home=tmp_path, api_key="x", api_base_url="https://example", request_timeout_sec=20)


def test_health_resource_reports_ok(tmp_path: Path) -> None:
    provider = ResourceProvider(_settings(tmp_path))
    health = provider.get_health()

    assert health["status"] == "ok"
    assert health["has_api_key"] is True
    assert health["issues"] == []


def test_health_resource_reports_missing_api_key(tmp_path: Path) -> None:
    settings = Settings(recite_home=tmp_path, api_key=None, api_base_url="https://example", request_timeout_sec=20)
    provider = ResourceProvider(settings)
    health = provider.get_health()

    assert health["status"] == "degraded"
    assert health["has_api_key"] is False
    assert "missing_api_key" in health["issues"]


def test_memory_resource_reads_file(tmp_path: Path) -> None:
    memory_file = tmp_path / "long_term_memory.md"
    memory_file.write_text("hello", encoding="utf-8")
    provider = ResourceProvider(_settings(tmp_path))

    assert provider.get_memory_text() == "hello"



from __future__ import annotations

from pathlib import Path

import pytest

from recite_mcp.api_client import ApiClient, ApiClientError
from recite_mcp.config import Settings


# ---------------------------------------------------------------------------
# Test helpers
# ---------------------------------------------------------------------------

class _Response:
    def __init__(
        self,
        status_code: int,
        payload: dict | None = None,
        *,
        text: str = "",
        headers: dict[str, str] | None = None,
    ) -> None:
        import json
        self.status_code = status_code
        self._payload = payload
        self.text = text or (json.dumps(payload) if payload is not None else "")
        self.headers = headers or {"Content-Type": "application/json"}

    def json(self) -> dict:
        if self._payload is None:
            raise ValueError("No JSON payload configured.")
        return self._payload


class _Session:
    def __init__(self, response: _Response | Exception) -> None:
        self._response = response
        self.last_method: str | None = None
        self.last_url: str | None = None
        self.last_kwargs: dict | None = None

    def request(self, method: str, url: str, **kwargs):  # noqa: ANN001
        self.last_method = method
        self.last_url = url
        self.last_kwargs = kwargs
        if isinstance(self._response, Exception):
            raise self._response
        return self._response

    def post(self, *args, **kwargs):  # noqa: ANN002, ANN003
        return self.request("POST", args[0], **kwargs)

    def get(self, *args, **kwargs):  # noqa: ANN002, ANN003
        return self.request("GET", args[0], **kwargs)

    def patch(self, *args, **kwargs):  # noqa: ANN002, ANN003
        return self.request("PATCH", args[0], **kwargs)

    def delete(self, *args, **kwargs):  # noqa: ANN002, ANN003
        return self.request("DELETE", args[0], **kwargs)


def _settings(tmp_path: Path) -> Settings:
    return Settings(
        recite_home=tmp_path,
        api_key="re_test_123",
        api_base_url="https://recite.rivra.dev/apiV1/api/v1",
        request_timeout_sec=30,
    )


def _settings_no_key(tmp_path: Path) -> Settings:
    return Settings(
        recite_home=tmp_path,
        api_key=None,
        api_base_url="https://recite.rivra.dev/apiV1/api/v1",
        request_timeout_sec=30,
    )


def _ok(data: dict) -> dict:
    """Convenience: build a standard success envelope."""
    return {"success": True, "data": data}


# ---------------------------------------------------------------------------
# process_receipt
# ---------------------------------------------------------------------------

def test_process_receipt_success(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    payload = {
        "success": True,
        "data": {"vendor": "Store", "date": "2026-02-22", "total": 10.5, "tax": 0.5, "currency": "USD"},
        "meta": {"api_version": "v1"},
    }
    session = _Session(_Response(200, payload))
    client = ApiClient(_settings(tmp_path), session=session)

    receipt = client.process_receipt(image)

    assert receipt.vendor == "Store"
    assert receipt.total == 10.5
    assert session.last_url is not None
    assert session.last_url.endswith("/apiV1/api/v1/scan")
    assert session.last_kwargs is not None
    assert "image_base64" in session.last_kwargs["json"]
    assert session.last_kwargs["json"]["auto_save"] is False


def test_process_receipt_reads_extracted_data_shape(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    payload = {
        "success": True,
        "data": {
            "scan_id": "scan_123",
            "extracted_data": {
                "vendor": "Nested Store",
                "date": "2026-03-01",
                "amount": 18.25,
                "tax": 1.25,
                "currency": "USD",
                "category": "Food & Dining",
            },
        },
    }
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, payload)))

    receipt = client.process_receipt(image)

    assert receipt.vendor == "Nested Store"
    assert receipt.total == 18.25
    assert receipt.category == "Food & Dining"


def test_process_receipt_raises_on_http_error(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(500, {"error": "failed"})))

    with pytest.raises(ApiClientError):
        client.process_receipt(image)


def test_process_receipt_raises_when_success_is_false(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake")
    payload = {"success": False, "error": {"message": "invalid image"}}
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, payload)))

    with pytest.raises(ApiClientError):
        client.process_receipt(image)


def test_process_receipt_raises_when_file_missing(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="does not exist"):
        client.process_receipt(tmp_path / "nonexistent.jpg")


# ---------------------------------------------------------------------------
# scan_receipt — input validation
# ---------------------------------------------------------------------------

def test_scan_receipt_supports_local_file_and_ephemeral_mode(tmp_path: Path) -> None:
    image = tmp_path / "receipt.png"
    image.write_bytes(b"fake")
    payload = {
        "success": True,
        "data": {
            "scan_id": "scan_ephemeral",
            "summary": {"confidence_band": "high", "auto_save_eligible": True},
        },
        "meta": {"request_id": "req_123"},
    }
    session = _Session(_Response(200, payload))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.scan_receipt(file_path=image, ephemeral=True)

    assert result["scan_id"] == "scan_ephemeral"
    assert session.last_method == "POST"
    assert session.last_url is not None
    assert session.last_url.endswith("/apiV1/api/v1/scan")
    assert session.last_kwargs is not None
    assert session.last_kwargs["json"]["ephemeral"] is True
    assert "image_base64" in session.last_kwargs["json"]


def test_scan_receipt_rejects_ephemeral_auto_save_combo(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="ephemeral"):
        client.scan_receipt(raw_text="receipt text", ephemeral=True, auto_save=True, project_id="proj_123")


def test_scan_receipt_rejects_auto_save_without_project_id(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="project_id"):
        client.scan_receipt(raw_text="receipt text", auto_save=True)


def test_scan_receipt_rejects_non_https_image_url(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="https"):
        client.scan_receipt(image_url="http://example.com/receipt.jpg")


def test_scan_receipt_rejects_zero_inputs(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.scan_receipt()


def test_scan_receipt_rejects_multiple_inputs(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.scan_receipt(image_url="https://example.com/r.jpg", raw_text="text")


def test_scan_receipt_with_image_url(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"scan_id": "scan_url"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.scan_receipt(image_url="https://example.com/receipt.jpg")

    assert result["scan_id"] == "scan_url"
    assert session.last_kwargs["json"]["image_url"] == "https://example.com/receipt.jpg"
    assert "image_base64" not in session.last_kwargs["json"]


def test_scan_receipt_with_image_base64(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"scan_id": "scan_b64"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.scan_receipt(image_base64="abc123==")

    assert result["scan_id"] == "scan_b64"
    assert session.last_kwargs["json"]["image_base64"] == "abc123=="


def test_scan_receipt_with_raw_text(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"scan_id": "scan_text"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.scan_receipt(raw_text="Starbucks\nTotal: $4.50")

    assert result["scan_id"] == "scan_text"
    assert session.last_kwargs["json"]["raw_text"] == "Starbucks\nTotal: $4.50"


def test_scan_receipt_guesses_image_type_from_file_extension(tmp_path: Path) -> None:
    image = tmp_path / "photo.png"
    image.write_bytes(b"fake")
    session = _Session(_Response(200, _ok({"scan_id": "s"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.scan_receipt(file_path=image)

    assert session.last_kwargs["json"].get("image_type") == "image/png"


def test_scan_receipt_raises_when_file_missing(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="does not exist"):
        client.scan_receipt(file_path=tmp_path / "missing.jpg")


def test_scan_receipt_auto_save_sends_project_id(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"scan_id": "s", "transaction_id": "txn_1"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.scan_receipt(
        raw_text="receipt", auto_save=True, project_id="proj_abc"
    )

    assert result["transaction_id"] == "txn_1"
    assert session.last_kwargs["json"]["auto_save"] is True
    assert session.last_kwargs["json"]["project_id"] == "proj_abc"


# ---------------------------------------------------------------------------
# CRUD — transactions
# ---------------------------------------------------------------------------

def test_list_transactions_passes_filter_params(tmp_path: Path) -> None:
    session = _Session(
        _Response(
            200,
            _ok({"transactions": [], "pagination": {"total": 0, "limit": 10, "offset": 5, "has_more": False}}),
        )
    )
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.list_transactions(start_date="2026-01-01", amount_min=10, offset=5, limit=10)

    assert result["pagination"]["offset"] == 5
    assert session.last_method == "GET"
    assert session.last_kwargs["params"] == {
        "start_date": "2026-01-01",
        "amount_min": 10,
        "offset": 5,
        "limit": 10,
    }


def test_list_transactions_omits_none_params(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"transactions": [], "pagination": {}})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.list_transactions(limit=10)

    assert "start_date" not in session.last_kwargs["params"]
    assert session.last_kwargs["params"] == {"limit": 10}


def test_create_transaction_sends_payload_dropping_none(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"transaction_id": "txn_new"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.create_transaction({
        "date": "2026-03-01",
        "amount": 42.50,
        "transaction_type": "Expense",
        "category": "Food",
        "payment_method": "Card",
        "vendor": None,  # should be dropped
    })

    assert result["transaction_id"] == "txn_new"
    assert "vendor" not in session.last_kwargs["json"]
    assert session.last_method == "POST"


def test_get_transaction_sends_correct_url(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"transaction_id": "txn_abc"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.get_transaction("txn_abc")

    assert result["transaction_id"] == "txn_abc"
    assert session.last_method == "GET"
    assert session.last_url is not None
    assert session.last_url.endswith("/transactions/txn_abc")


def test_update_transaction_sends_patch_dropping_none(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"transaction_id": "txn_abc", "amount": 99.0})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.update_transaction("txn_abc", {"amount": 99.0, "vendor": None})

    assert result["amount"] == 99.0
    assert session.last_method == "PATCH"
    assert "vendor" not in session.last_kwargs["json"]
    assert session.last_url is not None
    assert session.last_url.endswith("/transactions/txn_abc")


def test_delete_transaction_handles_no_content_response(tmp_path: Path) -> None:
    session = _Session(_Response(204, None, headers={"Content-Type": ""}))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.delete_transaction("txn_123")

    assert result == {"status": "deleted", "transaction_id": "txn_123"}
    assert session.last_method == "DELETE"
    assert session.last_url is not None
    assert session.last_url.endswith("/apiV1/api/v1/transactions/txn_123")


# ---------------------------------------------------------------------------
# import_transactions
# ---------------------------------------------------------------------------

def test_import_transactions_csv_sets_text_csv_request(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"total": 1, "imported": 1, "failed": 0})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.import_transactions(
        csv_text="date,amount,transaction_type,category,payment_method\n2026-03-01,12.50,Expense,Food,Card",
        all_or_nothing=True,
        project_id="proj_123",
    )

    assert result["imported"] == 1
    assert session.last_method == "POST"
    assert session.last_url is not None
    assert session.last_url.endswith("/apiV1/api/v1/import/transactions")
    assert session.last_kwargs["headers"]["Content-Type"] == "text/csv"
    assert session.last_kwargs["data"].startswith("date,amount")
    assert session.last_kwargs["params"] == {"all_or_nothing": "true", "project_id": "proj_123"}


def test_import_transactions_csv_file_path_reads_from_disk(tmp_path: Path) -> None:
    csv_file = tmp_path / "txns.csv"
    csv_file.write_text("date,amount,transaction_type,category,payment_method\n2026-03-01,50.0,Expense,Food,Card")
    session = _Session(_Response(200, _ok({"total": 1, "imported": 1, "failed": 0})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.import_transactions(csv_file_path=csv_file, all_or_nothing=False)

    assert result["imported"] == 1
    assert session.last_method == "POST"
    assert session.last_url is not None
    assert session.last_url.endswith("/apiV1/api/v1/import/transactions")
    assert session.last_kwargs["headers"]["Content-Type"] == "text/csv"
    # data is a file-like object (may be closed after context exit but ref still held)
    assert hasattr(session.last_kwargs["data"], "read")
    assert session.last_kwargs["params"] == {"all_or_nothing": "false"}


def test_import_transactions_json_sends_application_json(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"total": 2, "imported": 2, "failed": 0})))
    client = ApiClient(_settings(tmp_path), session=session)

    txns = [{"date": "2026-03-01", "amount": 10.0}]
    result = client.import_transactions(transactions=txns, project_id="proj_x")

    assert result["imported"] == 2
    assert session.last_kwargs["json"] == {"transactions": txns, "project_id": "proj_x"}
    assert "params" not in session.last_kwargs or session.last_kwargs.get("params") is None


def test_import_transactions_requires_exactly_one_source(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.import_transactions(transactions=[{"date": "2026-03-01"}], csv_text="date,amount")


def test_import_transactions_rejects_zero_sources(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.import_transactions()


# ---------------------------------------------------------------------------
# submit_batch_scans
# ---------------------------------------------------------------------------

def test_submit_batch_scans_sends_correct_payload(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"job_id": "job_1", "status": "processing"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.submit_batch_scans(
        items=[{"image_url": "https://example.com/r.jpg"}],
        auto_save=True,
        project_id="proj_x",
    )

    assert result["job_id"] == "job_1"
    assert session.last_method == "POST"
    assert session.last_url is not None
    assert session.last_url.endswith("/batch/scans")
    assert session.last_kwargs["json"]["auto_save"] is True
    assert session.last_kwargs["json"]["project_id"] == "proj_x"


def test_submit_batch_scans_converts_file_path_to_base64(tmp_path: Path) -> None:
    image = tmp_path / "receipt.jpg"
    image.write_bytes(b"fake-image-data")
    session = _Session(_Response(200, _ok({"job_id": "job_2", "status": "processing"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.submit_batch_scans(items=[{"file_path": str(image)}])

    sent_items = session.last_kwargs["json"]["items"]
    assert len(sent_items) == 1
    assert "image_base64" in sent_items[0]
    assert "file_path" not in sent_items[0]
    # base64 of b"fake-image-data"
    import base64
    assert sent_items[0]["image_base64"] == base64.b64encode(b"fake-image-data").decode("ascii")
    assert sent_items[0].get("image_type") == "image/jpeg"


def test_submit_batch_scans_rejects_non_https_url_in_item(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="https"):
        client.submit_batch_scans(items=[{"image_url": "http://example.com/r.jpg"}])


def test_submit_batch_scans_rejects_multiple_inputs_per_item(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.submit_batch_scans(items=[{
            "image_url": "https://example.com/r.jpg",
            "image_base64": "abc==",
        }])


def test_submit_batch_scans_rejects_zero_inputs_per_item(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="exactly one"):
        client.submit_batch_scans(items=[{"metadata": {"note": "oops"}}])


def test_submit_batch_scans_rejects_non_dict_item(tmp_path: Path) -> None:
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="object"):
        client.submit_batch_scans(items=["https://example.com/r.jpg"])  # type: ignore[list-item]


# ---------------------------------------------------------------------------
# batch status / results
# ---------------------------------------------------------------------------

def test_get_batch_scan_status_sends_correct_url(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"job_id": "job_99", "status": "completed"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.get_batch_scan_status("job_99")

    assert result["job_id"] == "job_99"
    assert session.last_method == "GET"
    assert session.last_url is not None
    assert session.last_url.endswith("/batch/scans/job_99")


def test_get_batch_scan_results_sends_correct_url(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"job_id": "job_99", "results": []})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.get_batch_scan_results("job_99")

    assert result["results"] == []
    assert session.last_url is not None
    assert session.last_url.endswith("/batch/scans/job_99/results")


# ---------------------------------------------------------------------------
# get_scan
# ---------------------------------------------------------------------------

def test_get_scan_sends_correct_url(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"scan_id": "scan_xyz"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.get_scan("scan_xyz")

    assert result["scan_id"] == "scan_xyz"
    assert session.last_method == "GET"
    assert session.last_url is not None
    assert session.last_url.endswith("/scan/scan_xyz")


# ---------------------------------------------------------------------------
# projects
# ---------------------------------------------------------------------------

def test_create_project_sends_name_and_description(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"project_id": "proj_new", "name": "Q1"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.create_project(name="Q1", description="Quarter 1")

    assert result["project_id"] == "proj_new"
    assert session.last_method == "POST"
    assert session.last_kwargs["json"] == {"name": "Q1", "description": "Quarter 1"}


def test_create_project_omits_none_description(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"project_id": "proj_2"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.create_project(name="Minimal")

    assert "description" not in session.last_kwargs["json"]


def test_update_project_sends_patch(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"project_id": "proj_1"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.update_project("proj_1", name="New Name", status="archived")

    assert session.last_method == "PATCH"
    assert session.last_url is not None
    assert session.last_url.endswith("/projects/proj_1")
    assert session.last_kwargs["json"] == {"name": "New Name", "status": "archived"}


def test_delete_project_returns_status_deleted(tmp_path: Path) -> None:
    session = _Session(_Response(204, None, headers={"Content-Type": ""}))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.delete_project("proj_1")

    assert result == {"status": "deleted", "project_id": "proj_1"}
    assert session.last_method == "DELETE"
    assert session.last_url is not None
    assert session.last_url.endswith("/projects/proj_1")


def test_list_projects_sends_status_filter(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"projects": [], "pagination": {}})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.list_projects(status="active", limit=20)

    assert session.last_method == "GET"
    assert session.last_kwargs["params"] == {"status": "active", "limit": 20}


# ---------------------------------------------------------------------------
# webhooks
# ---------------------------------------------------------------------------

def test_create_webhook_sends_url_and_events(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"webhook_id": "wh_1", "url": "https://my.app/wh"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.create_webhook(url="https://my.app/wh", events=["batch.completed"])

    assert result["webhook_id"] == "wh_1"
    assert session.last_kwargs["json"]["events"] == ["batch.completed"]
    assert "secret" not in session.last_kwargs["json"]


def test_create_webhook_includes_secret_when_provided(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"webhook_id": "wh_2"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.create_webhook(url="https://my.app/wh", events=["transaction.created"], secret="mysecret")

    assert session.last_kwargs["json"]["secret"] == "mysecret"


def test_delete_webhook_returns_status_deleted(tmp_path: Path) -> None:
    session = _Session(_Response(204, None, headers={"Content-Type": ""}))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.delete_webhook("wh_1")

    assert result == {"status": "deleted", "webhook_id": "wh_1"}
    assert session.last_url is not None
    assert session.last_url.endswith("/webhooks/wh_1")


# ---------------------------------------------------------------------------
# rules
# ---------------------------------------------------------------------------

def test_create_rule_sends_full_payload(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"rule_id": "rule_1"})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.create_rule(
        rule_type="vendor_category",
        condition={"vendor": "Starbucks"},
        action={"set_category": "Coffee"},
        priority=1,
    )

    assert result["rule_id"] == "rule_1"
    assert session.last_kwargs["json"]["priority"] == 1
    assert session.last_kwargs["json"]["condition"] == {"vendor": "Starbucks"}


def test_create_rule_omits_priority_when_none(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"rule_id": "rule_2"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.create_rule(
        rule_type="vendor_category",
        condition={"vendor": "Costco"},
        action={"set_category": "Wholesale"},
    )

    assert "priority" not in session.last_kwargs["json"]


def test_delete_rule_returns_status_deleted(tmp_path: Path) -> None:
    session = _Session(_Response(204, None, headers={"Content-Type": ""}))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.delete_rule("rule_1")

    assert result == {"status": "deleted", "rule_id": "rule_1"}


# ---------------------------------------------------------------------------
# summary, usage, export
# ---------------------------------------------------------------------------

def test_get_summary_sends_all_params(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"totals": {"income": 0, "expense": 0, "net": 0}})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.get_summary(period="last_30_days", group_by="category", project_id="proj_x")

    assert session.last_method == "GET"
    assert session.last_kwargs["params"] == {
        "period": "last_30_days",
        "group_by": "category",
        "project_id": "proj_x",
    }


def test_get_usage_sends_period_and_breakdown(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"quota": {"remaining": 50}})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.get_usage(period="current_month", breakdown="daily")

    assert result["quota"]["remaining"] == 50
    assert session.last_kwargs["params"] == {"period": "current_month", "breakdown": "daily"}


def test_export_transactions_returns_csv_body_when_api_responds_with_csv(tmp_path: Path) -> None:
    session = _Session(
        _Response(
            200,
            None,
            text="transaction_id,amount\n1,42.50\n",
            headers={"Content-Type": "text/csv"},
        )
    )
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.export_transactions(format="csv", filters={"project_id": "proj_123"})

    assert result == {
        "content_type": "text/csv",
        "body": "transaction_id,amount\n1,42.50\n",
    }


def test_export_transactions_json_returns_data(tmp_path: Path) -> None:
    session = _Session(_Response(200, _ok({"transactions": [], "total_count": 0})))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.export_transactions(format="json")

    assert result["total_count"] == 0
    assert session.last_kwargs["json"]["format"] == "json"


# ---------------------------------------------------------------------------
# _request — generic behaviour
# ---------------------------------------------------------------------------

def test_missing_api_key_raises_before_network(tmp_path: Path) -> None:
    # Session that would succeed if called — but it should never be reached
    client = ApiClient(_settings_no_key(tmp_path), session=_Session(_Response(200, _ok({}))))

    with pytest.raises(ApiClientError, match="Missing API key"):
        client.get_usage()


def test_request_wraps_network_exception_as_api_client_error(tmp_path: Path) -> None:
    import requests as req
    client = ApiClient(_settings(tmp_path), session=_Session(req.ConnectionError("timeout")))

    with pytest.raises(ApiClientError, match="Request failed"):
        client.get_usage()


def test_request_raises_on_400_with_structured_error(tmp_path: Path) -> None:
    payload = {"success": False, "error": {"code": "INVALID_REQUEST", "message": "bad date"}}
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(400, payload)))

    with pytest.raises(ApiClientError, match="bad date"):
        client.get_usage()


def test_request_raises_on_401_with_plain_error(tmp_path: Path) -> None:
    payload = {"success": False, "error": {"code": "INVALID_API_KEY", "message": "key revoked"}}
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(401, payload)))

    with pytest.raises(ApiClientError, match="key revoked"):
        client.get_scan("scan_1")


def test_request_handles_empty_body_non_204(tmp_path: Path) -> None:
    # Some endpoints may return 200 with empty body
    session = _Session(_Response(200, None, text="", headers={"Content-Type": "application/json"}))
    client = ApiClient(_settings(tmp_path), session=session)

    result = client.list_webhooks()

    assert result == {"status": "ok"}


def test_request_raises_on_success_false_with_200(tmp_path: Path) -> None:
    payload = {"success": False, "error": {"message": "quota exceeded"}}
    client = ApiClient(_settings(tmp_path), session=_Session(_Response(200, payload)))

    with pytest.raises(ApiClientError, match="quota exceeded"):
        client.scan_receipt(raw_text="text")


def test_request_raises_on_invalid_json_body(tmp_path: Path) -> None:
    # Response claims JSON content-type but body is not valid JSON
    response = _Response(200, None, text="not-json", headers={"Content-Type": "application/json"})
    # Patch json() to raise
    response._payload = None

    class _BrokenResponse(_Response):
        def json(self):
            raise ValueError("invalid json")

    br = _BrokenResponse(200, None, text="not-json", headers={"Content-Type": "application/json"})
    client = ApiClient(_settings(tmp_path), session=_Session(br))

    with pytest.raises(ApiClientError, match="Invalid JSON"):
        client.get_usage()


# ---------------------------------------------------------------------------
# URL path quoting
# ---------------------------------------------------------------------------

def test_path_segment_with_special_chars_is_url_encoded(tmp_path: Path) -> None:
    """IDs containing slashes or spaces must be percent-encoded in URLs."""
    session = _Session(_Response(200, _ok({"transaction_id": "txn/weird id"})))
    client = ApiClient(_settings(tmp_path), session=session)

    client.get_transaction("txn/weird id")

    assert session.last_url is not None
    assert "txn/weird" not in session.last_url  # raw slash would break routing
    assert "txn%2Fweird%20id" in session.last_url



from __future__ import annotations

import csv
import json
from dataclasses import asdict
from pathlib import Path
from uuid import uuid4

from recite_mcp.models import LedgerEntry, ReceiptRecord, utc_now_iso

_HEADERS = [
    "entry_id",
    "timestamp_utc",
    "entry_type",
    "vendor",
    "date",
    "total",
    "tax",
    "currency",
    "category",
    "source_file",
    "ref_entry_id",
    "correction_reason",
]


class LedgerRepository:
    def __init__(self, path: Path) -> None:
        self.path = path
        self.path.parent.mkdir(parents=True, exist_ok=True)

    def _ensure_file(self) -> None:
        if self.path.exists():
            return
        with self.path.open("w", newline="", encoding="utf-8") as f:
            writer = csv.DictWriter(f, fieldnames=_HEADERS)
            writer.writeheader()

    def append_receipt(self, receipt: ReceiptRecord, source_file: str) -> LedgerEntry:
        self._ensure_file()
        entry = LedgerEntry(
            entry_id=str(uuid4()),
            timestamp_utc=utc_now_iso(),
            entry_type="receipt",
            vendor=receipt.vendor,
            date=receipt.date,
            total=receipt.total,
            tax=receipt.tax,
            currency=receipt.currency,
            category=receipt.category or "Uncategorized",
            source_file=source_file,
        )
        self._append_entry(entry)
        return entry

    def add_correction(self, original_entry_id: str, corrected_fields: dict, reason: str) -> LedgerEntry:
        self._ensure_file()
        entry = LedgerEntry(
            entry_id=str(uuid4()),
            timestamp_utc=utc_now_iso(),
            entry_type="correction",
            vendor="",
            date="",
            total=0.0,
            tax=0.0,
            currency="",
            category="",
            source_file=json.dumps(corrected_fields, separators=(",", ":")),
            ref_entry_id=original_entry_id,
            correction_reason=reason,
        )
        self._append_entry(entry)
        return entry

    def _append_entry(self, entry: LedgerEntry) -> None:
        with self.path.open("a", newline="", encoding="utf-8") as f:
            writer = csv.DictWriter(f, fieldnames=_HEADERS)
            writer.writerow(
                {
                    "entry_id": entry.entry_id,
                    "timestamp_utc": entry.timestamp_utc,
                    "entry_type": entry.entry_type,
                    "vendor": entry.vendor,
                    "date": entry.date,
                    "total": f"{entry.total:.2f}",
                    "tax": f"{entry.tax:.2f}",
                    "currency": entry.currency,
                    "category": entry.category,
                    "source_file": entry.source_file,
                    "ref_entry_id": entry.ref_entry_id,
                    "correction_reason": entry.correction_reason,
                }
            )

    def read_all(self) -> list[LedgerEntry]:
        self._ensure_file()
        rows: list[LedgerEntry] = []
        with self.path.open("r", newline="", encoding="utf-8") as f:
            for row in csv.DictReader(f):
                rows.append(
                    LedgerEntry(
                        entry_id=row["entry_id"],
                        timestamp_utc=row["timestamp_utc"],
                        entry_type=row["entry_type"],
                        vendor=row["vendor"],
                        date=row["date"],
                        total=float(row["total"] or 0),
                        tax=float(row["tax"] or 0),
                        currency=row["currency"],
                        category=row["category"],
                        source_file=row["source_file"],
                        ref_entry_id=row.get("ref_entry_id", ""),
                        correction_reason=row.get("correction_reason", ""),
                    )
                )
        return rows

    def summarize(self, group_by: str = "vendor") -> dict[str, dict[str, float | int]]:
        result: dict[str, dict[str, float | int]] = {}
        for row in self.read_all():
            if row.entry_type != "receipt":
                continue
            key = getattr(row, group_by, "unknown") or "unknown"
            if key not in result:
                result[key] = {"count": 0, "total": 0.0}
            result[key]["count"] += 1
            result[key]["total"] = float(result[key]["total"]) + row.total
        return result

    def export_csv(self, output_path: Path) -> Path:
        output_path.parent.mkdir(parents=True, exist_ok=True)
        output_path.write_text(self.path.read_text(encoding="utf-8"), encoding="utf-8")
        return output_path

    def export_json(self, output_path: Path) -> Path:
        output_path.parent.mkdir(parents=True, exist_ok=True)
        payload = [asdict(row) for row in self.read_all()]
        output_path.write_text(json.dumps(payload, indent=2), encoding="utf-8")
        return output_path
